"""
author: kapil thangavelu <k_vertigo@objectrealms.net>
"""

from __future__ import nested_scopes
from OFS.SimpleItem import SimpleItem
from DateTime import DateTime
from Exceptions import NotFound

class ChangeCollection(SimpleItem):

    def __init__(self, id):
        self.id = id
        self.data = []
        self.update_position = None
        
    def addRecord(self, record_type, **kw):
        record = recordFactory.createRecord(record_type, **kw)
        self.data.append(record)

    def addRecordObj(self, record):
        self.data.append(record)

    def getRecords(self, offset=0):
        return tuple(self.data[offset:])

    def getLastRecord(self):
        return self.data[-1]

    def getFirstRecord(self):
        return self.data[0]

    def getRecordsFromUpdate(self):
        if update_position:
            return self.getRecords(offset=update_position)
        return self.getRecords()
    
    def walk(self, visitor):
        for i in self.data:
            visitor(i.__of__(self))
            
    def compactRecords(self):
        raise NotImplemented

    def updateState(self, context):
        raise NotImplemented

    def __len__(self):
        return len(self.data)

    def isCurrent(self):
        for r in self.data:
            if r.needs_update:
                return 1

class ChangeRecord(SimpleItem):

    needs_update = None
    ruid = None
    oid = None
    record_type = None
    conflicting = ()
    negating = ()
    updaters = ()
    
    def __init__(self, ruid, record_type, oid):
        self.ruid = ruid
        self.record_type = record_type
        self.oid = oid
        
    def getConflicting(self):
        return tuple(self.conflicting)
    
    def getNegating(self):
        return tuple(self.negating)
    
    def getUpdaters(self):
        return tuple(self.updaters)
    
    def update(self, context):
        raise NotImplemented

    def __eq__(self, other):
        return self.record_type == other and self.oid == other.oid
    
class RecordOIDCollection:

    def __init__(self, oid):
        self.data = {}
        self.conflicts = []
        self.oid = oid

    def addRecord(self, record):
        assert record.oid is self.oid
        if self.data.has_key(record.record_type):
            return None
        
        for c in self.conflicts:
            if isinstance(record, c):
                return None

        self.conflicts.extend(record.getConflicting())
        
        return self.data.setdefault(record.record_type, record)

    def getRecords(self):
        return self.data.values()

    def getRecordTypes(self):
        return self.data.keys()

    def getRecord(self, record_type):
        return self.data.get(record_type)

class RecordMultiOIDCollection:

    def __init__(self):
        self.data = {}

    def addRecord(self, record):
	pass
        
    
class UpdateSolver:
    """
    naive solver that just retrieves updates in order
    """

    def __init__(self, collection):
        self.collection = collection

    def getUpdateSet(self):
        return [r.getUpdaters() for r in self.collection.getRecords()]

class _RecordFactory:

    def __init__(self):
        self._registry = {}
        
    def registerRecordType(self, type_name, factory):
        self._registry[type_name] = factory

    def createRecord(self, type_name, *args, **kw):
        factory = self._registry.get(type_name)
        if factory is None:
            raise NotFound('no factory for record %s'%str(type_name))

recordFactory = _RecordFactory()
