"""
uuid implementation taken from peak.util.uuid, and then
slaughtered, smoked, and dressed. iotw, the original
implementation is much more featureful, and the code
here shouldn't be taken as a reflection of the original,
but i do want to give credit where credit is due. the code
here is a minimalistic version that is compatible with
python2.1

as per the original the LICENSE is PSF or ZPL
"""

import time
import sys
import os

from whrandom import random
from thread import allocate_lock

try:
    from sha import sha as hashfunc
except:
    from md5 import md5 as hashfunc

last_time = 0
offset = 0
clock_seq = '' # set latter during module init
last_hash = seed_hash = ""

_lock = allocate_lock()
acquire_lock = _lock.acquire
release_lock = _lock.release

def get_nodeid48():
    s = prng(6)
    s = [ord(x) for x in s]
    s[0] = s[0] | 128
    s = ["%02x" % x for x in s]
    s = ''.join(s)
    return s.lower()

def get_clockseq():
    return (rand16() & 0x3FFF) | 0x4000

def create_uuid():

    global last_time, offset

    nodeid = get_nodeid48()

    acquire_lock()
    try:
        t = time.time()
        if t==last_time: offset += (rand16() % 191) + 1
        else:  last_time, offset = t, 0
        ut = long(t * 10000000.0) + 0x01B21DD213814000L + offset        

    finally:
        release_lock()

    ut = "%015x" % ut
    
    uid_string = "%s-%s-1%s-%04x-%s" % (
        ut[7:], ut[3:7], ut[:3], clock_seq, nodeid
        )

    return uid_string


def prng_some():
    global last_hash, seed_hash
    
    try:
        import socket
        hn = socket.gethostname()
    except:
        hn = random()

    try:
        p = os.getpid()
    except:
        p = random()

    r = random()

    s = "%s%s%s%s%s" % (p, time.time(), id(time), id(os), last_hash)
    s = "%s%s%s%s%s" % (s, sys.version, sys.byteorder, hn, seed_hash)
    s = "%s%s%s%s%s" % (s, sys.getrefcount(None), id(None), id(r), id(p))
    s = "%s%s%s%s%s" % (s, id(time.time()), id(s), id(hn), r)

    lasthash = s = hashfunc(s).digest()

    return s

def prng(nbytes):
    """
    A PRNG that hopefully is better than just using whrandom.
    Don't trust this for crypto, though!
    """
    
    b = prng_some()
    while len(b) < nbytes:
        b += prng_some()

    return b[:nbytes]

def rand16():
    """16 bit unsigned random number"""
    
    s = prng(2)
    return (ord(s[0]) << 8) | ord(s[1])


clock_seq = get_clockseq()


if __name__ == '__main__':
    for i in range(20):
        print create_uuid()

